@@ -1,37 +1,39 @@
-use crate::client::temp::OwnedRequest;
 use crate::options::Broker;
+use crate::packet::SerializedPacket;
 use crate::state::{MqttConnectionStatus, MqttState};
-use crate::{EventError, MqttOptions, NetworkError, Notification};
+use crate::{mqtt_log, EventError, MqttOptions, NetworkError, Notification};
+use bbqueue::framed::FrameConsumer;
 use core::convert::Infallible;
+use core::ops::DerefMut;
 use core::ops::{Add, RangeTo};
 use embedded_nal::{AddrType, Dns, SocketAddr, TcpClientStack};
 use embedded_time::duration::Extensions;
 use embedded_time::duration::Milliseconds;
 use embedded_time::{Clock, Instant};
-use heapless::{spsc, String, Vec};
-use mqttrs::{decode_slice, encode_slice, Connect, Packet, Protocol, QoS};
+use heapless::{String, Vec};
+use mqttrust::encoding::v4::{decode_slice, encode_slice, Connect, Packet, Protocol};
 
-pub struct EventLoop<'a, 'b, S, O, const T: usize, const P: usize, const L: usize>
+pub struct EventLoop<'a, 'b, S, O, const L: usize>
 where
     O: Clock,
 {
     /// Current state of the connection
-    pub state: MqttState<Instant<O>, T, P>,
+    pub state: MqttState<Instant<O>>,
     /// Last outgoing packet time
     pub last_outgoing_timer: O,
     /// Options of the current mqtt connection
     pub options: MqttOptions<'b>,
     /// Request stream
-    pub requests: spsc::Consumer<'a, OwnedRequest<T, P>, L>,
+    pub requests: FrameConsumer<'a, L>,
     network_handle: NetworkHandle<S>,
 }
 
-impl<'a, 'b, S, O, const T: usize, const P: usize, const L: usize> EventLoop<'a, 'b, S, O, T, P, L>
+impl<'a, 'b, S, O, const L: usize> EventLoop<'a, 'b, S, O, L>
 where
     O: Clock,
 {
     pub fn new(
-        requests: spsc::Consumer<'a, OwnedRequest<T, P>, L>,
+        requests: FrameConsumer<'a, L>,
         outgoing_timer: O,
         options: MqttOptions<'b>,
     ) -> Self {
@@ -48,29 +50,41 @@ where
         &mut self,
         network: &mut N,
     ) -> nb::Result<bool, EventError> {
-        use EventError::*;
-
         // connect to the broker
-        if !self.network_handle.is_connected(network).map_err(|e| {
-            // On the socket error, clean it up and bail out.
-            self.network_handle.socket.take();
-            EventError::Network(e)
-        })? {
-            let (broker, port) = self.options.broker();
-            let (_hostname, socket_addr) = NetworkHandle::<S>::lookup_host(network, broker, port)
-                .map_err(EventError::Network)?;
-            self.network_handle
-                .connect(network, socket_addr)
-                .map_err(EventError::Network)?;
-            defmt::debug!("Network connected!");
+        match self.network_handle.is_connected(network) {
+            Ok(false) => {
+                // Socket is present, but not connected. Usually this implies
+                // that the socket is closed for writes. Disconnect to close &
+                // recycle the socket.
+                mqtt_log!(warn, "Socket cleanup!");
+                self.disconnect(network);
+                return Err(EventError::Network(NetworkError::SocketClosed).into());
+            }
+            Err(_) => {
+                // We have no socket present at all
+                let (broker, port) = self.options.broker();
+                let (_hostname, socket_addr) =
+                    NetworkHandle::<S>::lookup_host(network, broker, port)
+                        .map_err(EventError::Network)?;
+                self.network_handle
+                    .connect(network, socket_addr)
+                    .map_err(EventError::Network)?;
+                mqtt_log!(debug, "Network connected!");
 
-            self.state.connection_status = MqttConnectionStatus::Disconnected;
+                self.state.connection_status = MqttConnectionStatus::Disconnected;
+            }
+            Ok(true) => {
+                // Socket is there, and is connected. Proceed to make sure MQTT is connected
+            }
         }
 
         self.mqtt_connect(network).map_err(|e| {
             e.map(|e| {
-                if matches!(e, Network(_) | MqttState(_) | Timeout) {
-                    defmt::debug!("Disconnecting from {:?}", defmt::Debug2Format(&e));
+                if matches!(
+                    e,
+                    EventError::Network(_) | EventError::MqttState(_) | EventError::Timeout
+                ) {
+                    mqtt_log!(debug, "Disconnecting!");
                     self.disconnect(network);
                 }
                 e
@@ -79,15 +93,17 @@ where
     }
 
     fn should_handle_request(&mut self) -> bool {
-        let qos_space = self.state.outgoing_pub.len() < self.options.inflight();
+        let qos_space = self.state.outgoing_pub.len() < self.state.outgoing_pub.capacity();
 
-        let qos_0 = if let Some(OwnedRequest::Publish(p)) = self.requests.peek() {
-            p.qos == QoS::AtMostOnce
-        } else {
-            false
-        };
+        // TODO:
+        // let qos_0 = if let Some(_) = self.requests.read() {
+        //     p.qos == QoS::AtMostOnce
+        // } else {
+        //     false
+        // };
 
-        qos_0 || (self.requests.ready() && qos_space)
+        // qos_0 || (self.requests.ready() && qos_space)
+        qos_space
     }
 
     /// Selects an event from the client's requests, incoming packets from the
@@ -96,22 +112,26 @@ where
         &mut self,
         network: &mut N,
     ) -> nb::Result<Notification, EventError> {
-        let packet_buf = &mut [0u8; 1024];
-
         let now = self
             .last_outgoing_timer
             .try_now()
             .map_err(EventError::from)?;
 
+        // Handle a request
         if self.should_handle_request() {
-            // Handle a request
-            let request = unsafe { self.requests.dequeue_unchecked() };
-            let packet = self
-                .state
-                .handle_outgoing_request(request, packet_buf, &now)
-                .map_err(EventError::from)?;
-            self.network_handle.send(network, &packet)?;
-            return Err(nb::Error::WouldBlock);
+            if let Some(mut grant) = self.requests.read() {
+                let mut packet = SerializedPacket(grant.deref_mut());
+
+                match self.state.handle_outgoing_request(&mut packet, &now) {
+                    Ok(()) => {
+                        self.network_handle.send(network, packet.to_inner())?;
+                        grant.release();
+                        return Err(nb::Error::WouldBlock);
+                    }
+                    Err(crate::state::StateError::MaxMessagesInflight) => {}
+                    Err(e) => return Err(nb::Error::Other(e.into())),
+                }
+            }
         }
 
         if self
@@ -125,7 +145,7 @@ where
                 .state
                 .handle_outgoing_packet(Packet::Pingreq)
                 .map_err(EventError::from)?;
-            self.network_handle.send(network, &packet)?;
+            self.network_handle.send_packet(network, &packet)?;
             self.state.last_ping_entry().insert(now);
             return Err(nb::Error::WouldBlock);
         }
@@ -137,20 +157,19 @@ where
             .map_err(|e| e.map(EventError::Network))?
             .decode(&mut self.state)?;
 
+        // Handle `ack` of newly received incoming packet, if relevant
         if let Some(packet) = packet {
-            self.network_handle.send(network, &packet)?;
+            self.network_handle.send_packet(network, &packet)?;
         }
 
         // By comparing the current time, select pending non-zero QoS publish
         // requests staying longer than the retry interval, and handle their
         // retrial.
-        for (pid, inflight) in self.state.retries(now, 50_000.milliseconds()) {
-            defmt::warn!("Retrying PID {:?}", pid);
-            let packet = inflight
-                .packet(*pid, packet_buf)
-                .map_err(EventError::from)?;
+        for (pid, inflight) in self.state.retries(now, 10_000.milliseconds()) {
+            mqtt_log!(warn, "Retrying PID {:?}", pid);
             // Update inflight's timestamp for later retrials
             inflight.last_touch_entry().insert(now);
+            let packet = inflight.packet(*pid).map_err(EventError::from)?;
             self.network_handle.send(network, &packet)?;
         }
 
@@ -173,10 +192,7 @@ where
         self.select_event(network).or_else(|e| match e {
             nb::Error::WouldBlock => Err(nb::Error::WouldBlock),
             nb::Error::Other(e) => {
-                defmt::debug!(
-                    "Disconnecting from an event error. {:?}",
-                    defmt::Debug2Format(&e)
-                );
+                mqtt_log!(debug, "Disconnecting from an event error");
                 self.disconnect(network);
                 Ok(Notification::Abort(e))
             }
@@ -197,7 +213,7 @@ where
         match self.state.connection_status {
             MqttConnectionStatus::Connected => Ok(false),
             MqttConnectionStatus::Disconnected => {
-                defmt::info!("MQTT connecting..");
+                mqtt_log!(info, "MQTT connecting..");
                 let now = self
                     .last_outgoing_timer
                     .try_now()
@@ -205,12 +221,11 @@ where
                 self.state.last_ping_entry().insert(now);
 
                 self.state.await_pingresp = false;
-                self.state.outgoing_pub.clear();
                 self.network_handle.rx_buf.init();
 
                 let (username, password) = self.options.credentials();
 
-                let connect = Connect {
+                let connect = Packet::Connect(Connect {
                     protocol: Protocol::MQTT311,
                     keep_alive: (self.options.keep_alive_ms() / 1000) as u16,
                     client_id: self.options.client_id(),
@@ -218,10 +233,10 @@ where
                     last_will: self.options.last_will(),
                     username,
                     password,
-                };
+                });
 
                 // mqtt connection with timeout
-                self.network_handle.send(network, &connect.into())?;
+                self.network_handle.send_packet(network, &connect)?;
                 self.state.handle_outgoing_connect();
                 Err(nb::Error::WouldBlock)
             }
@@ -257,7 +272,7 @@ where
 struct NetworkHandle<S> {
     /// Network socket
     socket: Option<S>,
-    tx_buf: Vec<u8, 1024>,
+    tx_buf: heapless::Vec<u8, 64>,
     rx_buf: PacketBuffer,
 }
 
@@ -271,7 +286,7 @@ impl<S> NetworkHandle<S> {
             Broker::Hostname(h) => {
                 let socket_addr = SocketAddr::new(
                     network.get_host_by_name(h, AddrType::IPv4).map_err(|_e| {
-                        defmt::info!("Failed to resolve IP!");
+                        mqtt_log!(info, "Failed to resolve IP!");
                         NetworkError::DnsLookupFailed
                     })?,
                     port,
@@ -281,7 +296,7 @@ impl<S> NetworkHandle<S> {
             Broker::IpAddr(ip) => {
                 let socket_addr = SocketAddr::new(ip, port);
                 let domain = network.get_host_by_address(ip).map_err(|_e| {
-                    defmt::info!("Failed to resolve hostname!");
+                    mqtt_log!(info, "Failed to resolve hostname!");
                     NetworkError::DnsLookupFailed
                 })?;
 
@@ -293,7 +308,7 @@ impl<S> NetworkHandle<S> {
     fn new() -> Self {
         Self {
             socket: None,
-            tx_buf: Vec::new(),
+            tx_buf: heapless::Vec::new(),
             rx_buf: PacketBuffer::new(),
         }
     }
@@ -304,10 +319,12 @@ impl<S> NetworkHandle<S> {
         &self,
         network: &mut N,
     ) -> Result<bool, NetworkError> {
-        self.socket
-            .as_ref()
-            .map_or(Ok(false), |socket| network.is_connected(&socket))
-            .map_err(|_e| NetworkError::SocketClosed)
+        match self.socket {
+            Some(ref socket) => network
+                .is_connected(socket)
+                .map_err(|_e| NetworkError::SocketClosed),
+            None => Err(NetworkError::SocketClosed),
+        }
     }
 
     fn connect<N: Dns + TcpClientStack<TcpSocket = S>>(
@@ -323,26 +340,49 @@ impl<S> NetworkHandle<S> {
             Some(socket) => socket,
         };
 
-        nb::block!(network.connect(socket, socket_addr)).map_err(|_| NetworkError::SocketConnect)
+        nb::block!(network.connect(socket, socket_addr)).map_err(|_| {
+            self.socket.take();
+            NetworkError::SocketConnect
+        })
+    }
+
+    pub fn send_packet<'d, N: TcpClientStack<TcpSocket = S>>(
+        &mut self,
+        network: &mut N,
+        pkt: &Packet,
+    ) -> Result<usize, EventError> {
+        self.tx_buf.clear();
+        self.tx_buf
+            .resize_default(self.tx_buf.capacity())
+            .unwrap_or_else(|()| unreachable!("Input length equals to the current capacity."));
+
+        let size = encode_slice(&pkt, self.tx_buf.as_mut()).map_err(EventError::Encoding)?;
+
+        let socket = self
+            .socket
+            .as_mut()
+            .ok_or(EventError::Network(NetworkError::NoSocket))?;
+
+        let length = nb::block!(network.send(socket, &self.tx_buf[..size])).map_err(|_| {
+            mqtt_log!(error, "[send] NetworkError::Write");
+            EventError::Network(NetworkError::Write)
+        })?;
+
+        Ok(length)
     }
 
     pub fn send<'d, N: TcpClientStack<TcpSocket = S>>(
         &mut self,
         network: &mut N,
-        pkt: &Packet<'d>,
+        pkt: &[u8],
     ) -> Result<usize, EventError> {
         let socket = self
             .socket
             .as_mut()
             .ok_or(EventError::Network(NetworkError::NoSocket))?;
-        let capacity = self.tx_buf.capacity();
-        self.tx_buf.clear();
-        self.tx_buf
-            .resize(capacity, 0x00_u8)
-            .unwrap_or_else(|()| unreachable!("Input length equals to the current capacity."));
-        let size = encode_slice(&pkt, self.tx_buf.as_mut())?;
-        let length = nb::block!(network.send(socket, &self.tx_buf[..size])).map_err(|_| {
-            defmt::error!("[send] NetworkError::Write");
+
+        let length = nb::block!(network.send(socket, &pkt)).map_err(|_| {
+            mqtt_log!(error, "[send] NetworkError::Write");
             EventError::Network(NetworkError::Write)
         })?;
 
@@ -364,6 +404,7 @@ impl<S> NetworkHandle<S> {
 /// A placeholder that keeps a buffer and constructs a packet incrementally.
 /// Given that underlying `TcpClientStack` throws `WouldBlock` in a non-blocking
 /// manner, its packet construction won't block either.
+#[derive(Debug)]
 struct PacketBuffer {
     range: RangeTo<usize>,
     buffer: Vec<u8, 1024>,
@@ -415,7 +456,7 @@ impl PacketBuffer {
             if matches!(e, nb::Error::WouldBlock) {
                 nb::Error::WouldBlock
             } else {
-                defmt::error!("[receive] NetworkError::Read");
+                mqtt_log!(error, "[receive] NetworkError::Read");
                 nb::Error::Other(NetworkError::Read)
             }
         })?;
@@ -472,9 +513,9 @@ impl<'a> PacketDecoder<'a> {
             .into()
     }
 
-    fn decode<TIM, const T: usize, const P: usize>(
+    fn decode<TIM>(
         mut self,
-        state: &mut MqttState<TIM, T, P>,
+        state: &mut MqttState<TIM>,
     ) -> nb::Result<(Option<Notification>, Option<Packet<'static>>), EventError>
     where
         TIM: Add<Milliseconds, Output = TIM> + PartialOrd + Copy,
@@ -483,7 +524,7 @@ impl<'a> PacketDecoder<'a> {
         match decode_slice(buffer) {
             Err(e) => {
                 self.is_err.replace(true);
-                defmt::error!("Packet decode error! {:?}", defmt::Debug2Format(&e));
+                mqtt_log!(error, "Packet decode error!");
 
                 Err(EventError::Encoding(e).into())
             }
@@ -517,16 +558,15 @@ impl<'a> Drop for PacketDecoder<'a> {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::state::{Inflight, StartTime};
-    use core::convert::TryInto;
+    use crate::state::{Inflight, StartTime, BOXED_PUBLISH};
+    use bbqueue::BBBuffer;
     use embedded_time::clock::Error;
     use embedded_time::duration::Milliseconds;
     use embedded_time::fraction::Fraction;
     use embedded_time::Instant;
-    use heapless::spsc::Queue;
-    use mqttrs::Error::InvalidConnectReturnCode;
-    use mqttrs::{Connack, ConnectReturnCode, Pid, Publish, QosPid};
-    use mqttrust::PublishRequest;
+    use heapless::pool::singleton::Pool;
+    use mqttrust::encoding::v4::{Connack, ConnectReturnCode, Error as EncodingError, Pid};
+    use mqttrust::{Publish, QoS};
 
     #[derive(Debug)]
     struct ClockMock {
@@ -621,7 +661,11 @@ mod tests {
 
     #[test]
     fn success_receive_multiple_packets() {
-        let mut state = MqttState::<Milliseconds, 128, 512>::new();
+        let mut state = MqttState::<Milliseconds>::new();
+        const LEN: usize = 1024 * 10;
+        static mut PUBLISH_MEM: [u8; LEN] = [0u8; LEN];
+        BOXED_PUBLISH::grow(unsafe { &mut PUBLISH_MEM });
+
         let mut rx_buf = PacketBuffer::new();
         let connack = Connack {
             session_present: false,
@@ -629,7 +673,8 @@ mod tests {
         };
         let publish = Publish {
             dup: false,
-            qospid: QosPid::AtLeastOnce(Pid::new()),
+            qos: QoS::AtLeastOnce,
+            pid: Some(Pid::new()),
             retain: false,
             topic_name: "test/topic",
             payload: &[0xff; 1003],
@@ -647,6 +692,9 @@ mod tests {
         assert_eq!(n, Some(Notification::ConnAck));
         assert_eq!(p, None);
 
+        let mut pkg = SerializedPacket(&mut rx_buf.buffer[rx_buf.range]);
+        pkg.set_pid(Pid::new()).unwrap();
+
         // Decode the second Publish packet on the Connected state.
         assert_eq!(state.connection_status, MqttConnectionStatus::Connected);
         let (n, p) = PacketDecoder::new(&mut rx_buf).decode(&mut state).unwrap();
@@ -662,7 +710,11 @@ mod tests {
 
     #[test]
     fn failure_receive_multiple_packets() {
-        let mut state = MqttState::<Milliseconds, 128, 512>::new();
+        let mut state = MqttState::<Milliseconds>::new();
+        const LEN: usize = 1024 * 10;
+        static mut PUBLISH_MEM: [u8; LEN] = [0u8; LEN];
+        BOXED_PUBLISH::grow(unsafe { &mut PUBLISH_MEM });
+
         let mut rx_buf = PacketBuffer::new();
         let connack_malformed = Connack {
             session_present: false,
@@ -670,7 +722,8 @@ mod tests {
         };
         let publish = Publish {
             dup: false,
-            qospid: QosPid::AtLeastOnce(Pid::new()),
+            qos: QoS::AtLeastOnce,
+            pid: Some(Pid::new()),
             retain: false,
             topic_name: "test/topic",
             payload: &[0xff; 1003],
@@ -690,7 +743,10 @@ mod tests {
         match PacketDecoder::new(&mut rx_buf).decode(&mut state) {
             Ok((_, _)) | Err(nb::Error::WouldBlock) => panic!(),
             Err(nb::Error::Other(e)) => {
-                assert_eq!(e, EventError::Encoding(InvalidConnectReturnCode(6)))
+                assert_eq!(
+                    e,
+                    EventError::Encoding(EncodingError::InvalidConnectReturnCode(6))
+                )
             }
         }
         assert_eq!(state.connection_status, MqttConnectionStatus::Handshake);
@@ -700,15 +756,15 @@ mod tests {
 
     #[test]
     fn retry_behaviour() {
-        static mut Q: Queue<OwnedRequest<128, 512>, 5> = Queue::new();
+        static mut Q: BBBuffer<1024> = BBBuffer::new();
 
         let mut network = MockNetwork {
             should_fail_read: false,
             should_fail_write: false,
         };
 
-        let (_p, c) = unsafe { Q.split() };
-        let mut event = EventLoop::<(), _, 128, 512, 5>::new(
+        let (_p, c) = unsafe { Q.try_split_framed().unwrap() };
+        let mut event = EventLoop::<(), _, 1024>::new(
             c,
             ClockMock { time: 0 },
             MqttOptions::new("client", Broker::Hostname(""), 8883),
@@ -716,64 +772,26 @@ mod tests {
 
         let now = StartTime::from(0);
 
-        event
-            .state
-            .outgoing_pub
-            .insert(
-                2,
-                Inflight::new(
-                    now,
-                    PublishRequest {
-                        dup: false,
-                        qos: QoS::AtLeastOnce,
-                        retain: false,
-                        topic_name: "some/topic/name2",
-                        payload: &[],
-                    }
-                    .try_into()
-                    .unwrap(),
-                ),
-            )
-            .unwrap();
+        let topic = "hello/world";
+        let payload = &[1, 2, 3];
+
+        let publish = Publish {
+            qos: QoS::AtLeastOnce,
+            pid: Some(Pid::new()),
+            payload,
+            dup: false,
+            retain: false,
+            topic_name: topic,
+        };
+
+        let mut rx_buf = PacketBuffer::new();
+        let publish_len = encode_slice(&Packet::from(publish.clone()), rx_buf.buffer()).unwrap();
+        rx_buf.range.end += publish_len;
 
         event
             .state
             .outgoing_pub
-            .insert(
-                3,
-                Inflight::new(
-                    now,
-                    PublishRequest {
-                        dup: false,
-                        qos: QoS::AtLeastOnce,
-                        retain: false,
-                        topic_name: "some/topic/name3",
-                        payload: &[],
-                    }
-                    .try_into()
-                    .unwrap(),
-                ),
-            )
-            .unwrap();
-
-        event
-            .state
-            .outgoing_pub
-            .insert(
-                4,
-                Inflight::new(
-                    now,
-                    PublishRequest {
-                        dup: false,
-                        qos: QoS::AtLeastOnce,
-                        retain: false,
-                        topic_name: "some/topic/name4",
-                        payload: &[],
-                    }
-                    .try_into()
-                    .unwrap(),
-                ),
-            )
+            .insert(2, Inflight::new(now, &rx_buf.buffer))
             .unwrap();
 
         event.state.connection_status = MqttConnectionStatus::Handshake;
