@@ -1,14 +1,20 @@
-use crate::client::temp::{OwnedPublishRequest, OwnedRequest};
-use crate::{Notification, Subscribe, SubscribeRequest, UnsubscribeRequest};
+use crate::mqtt_log;
+use crate::packet::SerializedPacket;
+use crate::Notification;
+use crate::PublishNotification;
 use core::convert::TryInto;
 use core::ops::Add;
-use embedded_time::duration::{Generic, Milliseconds};
-use embedded_time::{Clock, Instant};
+use embedded_time::duration::Milliseconds;
+use heapless::{
+    pool,
+    pool::singleton::Pool,
+};
 use heapless::{FnvIndexMap, FnvIndexSet, IndexMap, IndexSet};
-use mqttrs::*;
+use mqttrust::encoding::v4::*;
 
 #[allow(unused)]
-#[derive(Debug, Clone, Copy, PartialEq, Eq, defmt::Format)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+#[cfg_attr(feature = "defmt-impl", derive(defmt::Format))]
 pub enum MqttConnectionStatus {
     Handshake,
     Connected,
@@ -16,6 +22,7 @@ pub enum MqttConnectionStatus {
 }
 
 #[derive(Debug, PartialEq)]
+#[cfg_attr(feature = "defmt-impl", derive(defmt::Format))]
 pub enum StateError {
     /// Broker's error reply to client's connect packet
     Connect(ConnectReturnCode),
@@ -33,8 +40,11 @@ pub enum StateError {
     MaxMessagesInflight,
     /// Non-zero QoS publications require PID
     PidMissing,
+    InvalidHeader,
 }
 
+pool!(BOXED_PUBLISH: PublishNotification);
+
 /// State of the mqtt connection.
 ///
 /// Methods will just modify the state of the object without doing any network
@@ -47,7 +57,7 @@ pub enum StateError {
 /// **Generics**:
 /// - O: The output timer used for keeping track of keep-alive ping-pongs. Must
 ///   implement the [`embedded_hal::timer::CountDown`] trait
-pub struct MqttState<TIM, const T: usize, const P: usize> {
+pub struct MqttState<TIM> {
     /// Connection status
     pub connection_status: MqttConnectionStatus,
     /// Status of last ping
@@ -55,15 +65,15 @@ pub struct MqttState<TIM, const T: usize, const P: usize> {
     /// Packet id of the last outgoing packet
     pub last_pid: Pid,
     /// Outgoing QoS 1, 2 publishes which aren't acked yet
-    pub(crate) outgoing_pub: FnvIndexMap<u16, Inflight<TIM, T, P>, 4>,
+    pub(crate) outgoing_pub: FnvIndexMap<u16, Inflight<TIM, 1536>, 1>,
     /// Packet ids of released QoS 2 publishes
-    pub outgoing_rel: FnvIndexSet<u16, 4>,
+    pub outgoing_rel: FnvIndexSet<u16, 1>,
     /// Packet ids on incoming QoS 2 publishes
-    pub incoming_pub: FnvIndexSet<u16, 2>,
+    pub incoming_pub: FnvIndexSet<u16, 1>,
     last_ping: StartTime<TIM>,
 }
 
-impl<TIM, const T: usize, const P: usize> MqttState<TIM, T, P>
+impl<TIM> MqttState<TIM>
 where
     TIM: Add<Milliseconds, Output = TIM> + PartialOrd + Copy,
 {
@@ -71,6 +81,14 @@ where
     /// connection for persistent sessions while new state should
     /// instantiated for clean sessions
     pub fn new() -> Self {
+        const LEN: usize = core::mem::size_of::<PublishNotification>()
+            + core::mem::align_of::<PublishNotification>()
+            - core::mem::size_of::<PublishNotification>()
+                % core::mem::align_of::<PublishNotification>();
+
+        static mut PUBLISH_MEM: [u8; LEN] = [0u8; LEN];
+        BOXED_PUBLISH::grow(unsafe { &mut PUBLISH_MEM });
+
         MqttState {
             connection_status: MqttConnectionStatus::Disconnected,
             await_pingresp: false,
@@ -91,24 +109,24 @@ where
     ) -> Result<Packet<'b>, StateError> {
         match packet {
             Packet::Pingreq => self.handle_outgoing_ping(),
-            _ => unimplemented!(),
+            _ => unreachable!(),
         }
     }
 
     /// Consolidates handling of all outgoing mqtt packet logic. Returns a
     /// packet which should be put on to the network by the eventloop
-    pub fn handle_outgoing_request<'b>(
+    pub fn handle_outgoing_request(
         &mut self,
-        request: OwnedRequest<T, P>,
-        buf: &'b mut [u8],
+        request: &mut SerializedPacket<'_>,
         now: &TIM,
-    ) -> Result<Packet<'b>, StateError> {
-        match request {
-            OwnedRequest::Publish(publish) => self.handle_outgoing_publish(publish, buf, now),
-            OwnedRequest::Subscribe(subscribe) => self.handle_outgoing_subscribe(subscribe),
-            OwnedRequest::Unsubscribe(unsubscribe) => self.handle_outgoing_unsubscribe(unsubscribe),
-            _ => unimplemented!(),
+    ) -> Result<(), StateError> {
+        match request.header()?.typ {
+            PacketType::Publish => self.handle_outgoing_publish(request, now)?,
+            PacketType::Subscribe => request.set_pid(self.next_pid())?,
+            PacketType::Unsubscribe => request.set_pid(self.next_pid())?,
+            _ => unreachable!(),
         }
+        Ok(())
     }
 
     /// Consolidates handling of all incoming mqtt packets. Returns a
@@ -133,7 +151,7 @@ where
             Packet::Pubrel(pid) => self.handle_incoming_pubrel(pid),
             Packet::Pubcomp(pid) => self.handle_incoming_pubcomp(pid),
             _ => {
-                defmt::error!("Invalid incoming packet!");
+                mqtt_log!(error, "Invalid incoming packet!");
                 Ok((None, None))
             }
         }
@@ -141,39 +159,29 @@ where
 
     /// Adds next packet identifier to QoS 1 and 2 publish packets and returns
     /// it by wrapping publish in packet
-    fn handle_outgoing_publish<'b>(
+    fn handle_outgoing_publish(
         &mut self,
-        request: OwnedPublishRequest<T, P>,
-        buf: &'b mut [u8],
+        request: &mut SerializedPacket<'_>,
         now: &TIM,
-    ) -> Result<Packet<'b>, StateError> {
-        let pid = match request.qos {
-            QoS::AtMostOnce => None,
+    ) -> Result<(), StateError> {
+        match request.header()?.qos {
+            QoS::AtMostOnce => {}
             QoS::AtLeastOnce => {
                 let pid = self.next_pid();
                 self.outgoing_pub
-                    .insert(
-                        pid.get(),
-                        Inflight::new(StartTime::new(*now), request.clone()),
-                    )
+                    .insert(pid.get(), Inflight::new(StartTime::new(*now), &request.0))
                     .map_err(|_| StateError::MaxMessagesInflight)?;
-                pid.into()
+                request.set_pid(pid)?;
             }
             QoS::ExactlyOnce => {
                 let pid = self.next_pid();
                 self.outgoing_pub
-                    .insert(
-                        pid.get(),
-                        Inflight::new(StartTime::new(*now), request.clone()),
-                    )
+                    .insert(pid.get(), Inflight::new(StartTime::new(*now), &request.0))
                     .map_err(|_| StateError::MaxMessagesInflight)?;
-                pid.into()
+                request.set_pid(pid)?;
             }
-        };
-
-        pid.into_iter()
-            .fold(PublishPacketBuilder::new(buf), PublishPacketBuilder::pid)
-            .build(&request)
+        }
+        Ok(())
     }
 
     /// Iterates through the list of stored publishes and removes the publish
@@ -192,18 +200,20 @@ where
             let notification = Some(Notification::Puback(pid));
             Ok((notification, request))
         } else {
-            defmt::error!("Unsolicited puback packet: {:?}", pid.get());
-            Err(StateError::Unsolicited)
+            mqtt_log!(error, "Unsolicited puback packet: {:?}", pid.get());
+            // Err(StateError::Unsolicited)
+            Ok((None, None))
         }
     }
 
-    fn handle_incoming_suback(
+    fn handle_incoming_suback<'a>(
         &mut self,
-        suback: Suback,
+        _suback: Suback<'a>,
     ) -> Result<(Option<Notification>, Option<Packet<'static>>), StateError> {
-        let request = None;
-        let notification = Some(Notification::Suback(suback));
-        Ok((notification, request))
+        // TODO:
+        // let request = None;
+        // let notification = Some(Notification::Suback(suback));
+        Ok((None, None))
     }
 
     fn handle_incoming_unsuback(
@@ -224,7 +234,7 @@ where
         pid: Pid,
     ) -> Result<(Option<Notification>, Option<Packet<'static>>), StateError> {
         if self.outgoing_pub.contains_key(&pid.get()) {
-            let _publish = self.outgoing_pub.remove(&pid.get());
+            self.outgoing_pub.remove(&pid.get());
             self.outgoing_rel
                 .insert(pid.get())
                 .map_err(|_| StateError::InvalidState)?;
@@ -233,8 +243,9 @@ where
             let notification = Some(Notification::Pubrec(pid));
             Ok((notification, reply))
         } else {
-            defmt::error!("Unsolicited pubrec packet: {:?}", pid.get());
-            Err(StateError::Unsolicited)
+            mqtt_log!(error, "Unsolicited pubrec packet: {:?}", pid.get());
+            // Err(StateError::Unsolicited)
+            Ok((None, None))
         }
     }
 
@@ -244,20 +255,22 @@ where
         &mut self,
         publish: Publish<'b>,
     ) -> Result<(Option<Notification>, Option<Packet<'static>>), StateError> {
-        let qospid = publish.qospid;
-        let notification = Notification::Publish(publish.try_into()?);
+        let qospid = (publish.qos, publish.pid);
+        let boxed_publish = BOXED_PUBLISH::alloc().unwrap();
+        let notification = Notification::Publish(boxed_publish.init(publish.try_into().unwrap()));
 
         let request = match qospid {
-            QosPid::AtMostOnce => None,
-            QosPid::AtLeastOnce(pid) => Some(Packet::Puback(pid)),
-            QosPid::ExactlyOnce(pid) => {
+            (QoS::AtMostOnce, _) => None,
+            (QoS::AtLeastOnce, Some(pid)) => Some(Packet::Puback(pid)),
+            (QoS::ExactlyOnce, Some(pid)) => {
                 self.incoming_pub.insert(pid.get()).map_err(|_| {
-                    defmt::error!("Failed to insert incoming pub!");
+                    mqtt_log!(error, "Failed to insert incoming pub!");
                     StateError::InvalidState
                 })?;
 
                 Some(Packet::Pubrec(pid))
             }
+            _ => return Err(StateError::InvalidHeader),
         };
         Ok((Some(notification), request))
     }
@@ -271,8 +284,9 @@ where
             let reply = Packet::Pubcomp(pid);
             Ok((None, Some(reply)))
         } else {
-            defmt::error!("Unsolicited pubrel packet: {:?}", pid.get());
-            Err(StateError::Unsolicited)
+            mqtt_log!(error, "Unsolicited pubrel packet: {:?}", pid.get());
+            // Err(StateError::Unsolicited)
+            Ok((None, None))
         }
     }
 
@@ -286,8 +300,9 @@ where
             let reply = None;
             Ok((notification, reply))
         } else {
-            defmt::error!("Unsolicited pubcomp packet: {:?}", pid.get());
-            Err(StateError::Unsolicited)
+            mqtt_log!(error, "Unsolicited pubcomp packet: {:?}", pid.get());
+            // Err(StateError::Unsolicited)
+            Ok((None, None))
         }
     }
 
@@ -297,13 +312,13 @@ where
     fn handle_outgoing_ping<'b>(&mut self) -> Result<Packet<'b>, StateError> {
         // raise error if last ping didn't receive ack
         if self.await_pingresp {
-            defmt::error!("Error awaiting for last ping response");
+            mqtt_log!(error, "Error awaiting for last ping response");
             return Err(StateError::AwaitPingResp);
         }
 
         self.await_pingresp = true;
 
-        defmt::trace!("Pingreq");
+        mqtt_log!(trace, "Pingreq");
 
         Ok(Packet::Pingreq)
     }
@@ -312,32 +327,10 @@ where
         &mut self,
     ) -> Result<(Option<Notification>, Option<Packet<'static>>), StateError> {
         self.await_pingresp = false;
-        defmt::trace!("Pingresp");
+        mqtt_log!(trace, "Pingresp");
         Ok((None, None))
     }
 
-    fn handle_outgoing_subscribe<'b>(
-        &mut self,
-        subscribe_request: SubscribeRequest,
-    ) -> Result<Packet<'b>, StateError> {
-        defmt::trace!("Subscribe. Topics = ");
-        for topic in &subscribe_request.topics {
-            defmt::trace!("{=str}", topic.topic_path.as_str());
-        }
-        Ok(Subscribe::new(self.next_pid(), subscribe_request.topics).into())
-    }
-
-    fn handle_outgoing_unsubscribe<'b>(
-        &mut self,
-        unsubscribe_request: UnsubscribeRequest,
-    ) -> Result<Packet<'b>, StateError> {
-        defmt::trace!("Unsubscribe. Topics = ");
-        for topic in &unsubscribe_request.topics {
-            defmt::trace!("{=str}", topic.as_str());
-        }
-        Ok(Unsubscribe::new(self.next_pid(), unsubscribe_request.topics).into())
-    }
-
     pub(crate) fn handle_outgoing_connect(&mut self) {
         self.connection_status = MqttConnectionStatus::Handshake;
     }
@@ -347,14 +340,15 @@ where
             ConnectReturnCode::Accepted
                 if self.connection_status == MqttConnectionStatus::Handshake =>
             {
-                defmt::debug!("MQTT connected!");
+                mqtt_log!(debug, "MQTT connected!");
                 self.connection_status = MqttConnectionStatus::Connected;
                 Ok(())
             }
             ConnectReturnCode::Accepted
                 if self.connection_status != MqttConnectionStatus::Handshake =>
             {
-                defmt::error!(
+                mqtt_log!(
+                    error,
                     "Invalid state. Expected = {:?}, Current = {:?}",
                     MqttConnectionStatus::Handshake,
                     self.connection_status
@@ -363,7 +357,11 @@ where
                 Err(StateError::InvalidState)
             }
             code => {
-                defmt::error!("Connection failed. Connection error = {:?}", code as u8);
+                mqtt_log!(
+                    error,
+                    "Connection failed. Connection error = {:?}",
+                    code as u8
+                );
                 self.connection_status = MqttConnectionStatus::Disconnected;
                 Err(StateError::Connect(code))
             }
@@ -383,7 +381,7 @@ where
         &mut self,
         now: TIM,
         interval: Milliseconds,
-    ) -> impl Iterator<Item = (&u16, &mut Inflight<TIM, T, P>)> + '_ {
+    ) -> impl Iterator<Item = (&u16, &mut Inflight<TIM, 1536>)> + '_ {
         self.outgoing_pub
             .iter_mut()
             .filter(move |(_, inflight)| inflight.last_touch.has_elapsed(&now, interval))
@@ -429,10 +427,11 @@ where
     }
 }
 
-impl<O> defmt::Format for StartTime<Instant<O>>
+#[cfg(feature = "defmt-impl")]
+impl<O> defmt::Format for StartTime<embedded_time::Instant<O>>
 where
-    O: Clock,
-    Generic<O::T>: TryInto<Milliseconds>,
+    O: embedded_time::Clock,
+    embedded_time::duration::Generic<O::T>: TryInto<embedded_time::duration::Milliseconds>,
 {
     fn format(&self, fmt: defmt::Formatter) {
         let start_time = self
@@ -444,76 +443,29 @@ where
     }
 }
 
-struct PublishPacketBuilder<'a> {
-    pid: Option<Pid>,
-    buf: &'a mut [u8],
-}
-
-impl<'a> PublishPacketBuilder<'a> {
-    fn new(buf: &'a mut [u8]) -> Self {
-        Self { pid: None, buf }
-    }
-
-    fn pid(self, pid: Pid) -> Self {
-        Self {
-            pid: pid.into(),
-            ..self
-        }
-    }
-
-    fn build<const T: usize, const P: usize>(
-        self,
-        request: &OwnedPublishRequest<T, P>,
-    ) -> Result<Packet<'a>, StateError> {
-        let qospid = match request.qos {
-            QoS::AtMostOnce => QosPid::AtMostOnce,
-            QoS::AtLeastOnce => QosPid::AtLeastOnce(self.pid.ok_or(StateError::PidMissing)?),
-            QoS::ExactlyOnce => QosPid::ExactlyOnce(self.pid.ok_or(StateError::PidMissing)?),
-        };
-
-        // Construct topic name from the request and align on a buffer.
-        let topic_len = request.topic_name.len();
-        let payload_len = request.payload.len();
-
-        let (topic_buf, payload_buf) = self.buf.split_at_mut(topic_len);
-
-        topic_buf.copy_from_slice(request.topic_name.as_bytes());
-        let topic_name = core::str::from_utf8(topic_buf).map_err(|_| StateError::InvalidUtf8)?;
-
-        payload_buf[..payload_len].copy_from_slice(&request.payload);
-
-        Ok(Publish {
-            dup: request.dup,
-            qospid,
-            retain: request.retain,
-            topic_name,
-            payload: &payload_buf[..payload_len],
-        }
-        .into())
-    }
-}
-
 /// Client publication message data.
 #[derive(Debug)]
-pub(crate) struct Inflight<TIM, const T: usize, const P: usize> {
+pub(crate) struct Inflight<TIM, const L: usize> {
     /// A publish of non-zero QoS.
-    publish: OwnedPublishRequest<T, P>,
+    publish: heapless::Vec<u8, L>,
     /// A timestmap used for retry and expiry.
     last_touch: StartTime<TIM>,
-    // next_message_type, only relevant for Qos2, i.e., PUBREC, PUBREL or PUBCOMP.
 }
 
-impl<TIM, const T: usize, const P: usize> Inflight<TIM, T, P>
+impl<TIM, const L: usize> Inflight<TIM, L>
 where
     TIM: Add<Milliseconds, Output = TIM> + PartialOrd + Copy,
 {
-    pub(crate) fn new(last_touch: StartTime<TIM>, publish: OwnedPublishRequest<T, P>) -> Self {
+    pub(crate) fn new(last_touch: StartTime<TIM>, publish: &[u8]) -> Self {
         assert!(
-            !matches!(publish.qos, QoS::AtMostOnce),
+            !matches!(
+                decoder::Header::new(publish[0]).unwrap().qos,
+                QoS::AtMostOnce
+            ),
             "Only non-zero QoSs are allowed."
         );
         Self {
-            publish,
+            publish: heapless::Vec::from_slice(publish).unwrap(),
             last_touch,
         }
     }
@@ -523,27 +475,28 @@ where
     }
 }
 
-impl<TIM, const T: usize, const P: usize> Inflight<TIM, T, P> {
-    pub(crate) fn packet<'b>(&self, pid: u16, buf: &'b mut [u8]) -> Result<Packet<'b>, StateError> {
+impl<TIM, const L: usize> Inflight<TIM, L> {
+    pub(crate) fn packet<'b>(&'b mut self, pid: u16) -> Result<&'b [u8], StateError> {
         let pid = pid.try_into().map_err(|_| StateError::PayloadEncoding)?;
-        PublishPacketBuilder::new(buf).pid(pid).build(&self.publish)
+        let mut packet = SerializedPacket(self.publish.as_mut());
+        packet.set_pid(pid)?;
+        Ok(packet.to_inner())
     }
 }
 
 #[cfg(test)]
 mod test {
     use super::{
-        Clock, Milliseconds, MqttConnectionStatus, MqttState, Packet, StartTime, StateError,
+        Milliseconds, MqttConnectionStatus, MqttState, Packet, StartTime, StateError, BOXED_PUBLISH,
     };
-    use crate::client::temp::OwnedPublishRequest;
-    use crate::Notification;
+    use crate::{packet::SerializedPacket, Notification};
     use core::convert::TryFrom;
-    use core::convert::TryInto;
-    use embedded_time::duration::Extensions;
-    use embedded_time::Instant;
-    use heapless::{String, Vec};
-    use mqttrs::*;
-    use mqttrust::{PublishRequest, SubscribeRequest, UnsubscribeRequest};
+    use embedded_time::{duration::Extensions, Clock, Instant};
+    use heapless::pool::singleton::Pool;
+    use mqttrust::{
+        encoding::v4::{decode_slice, encode_slice, Pid},
+        Publish, QoS,
+    };
 
     impl<O> From<u32> for StartTime<Instant<O>>
     where
@@ -554,25 +507,19 @@ mod test {
         }
     }
 
-    fn build_outgoing_publish<'a>(qos: QoS) -> OwnedPublishRequest<128, 512> {
-        PublishRequest::new("hello/world", &[1, 2, 3])
-            .qos(qos)
-            .try_into()
-            .unwrap()
-    }
-
-    fn build_incoming_publish<'a>(qos: QoS, pid: u16) -> Publish<'a> {
+    fn build_publish<'a>(qos: QoS, pid: Option<u16>) -> Publish<'a> {
         let topic = "hello/world";
         let payload = &[1, 2, 3];
 
-        let qospid = match qos {
-            QoS::AtMostOnce => QosPid::AtMostOnce,
-            QoS::AtLeastOnce => QosPid::AtLeastOnce(Pid::try_from(pid).unwrap()),
-            QoS::ExactlyOnce => QosPid::ExactlyOnce(Pid::try_from(pid).unwrap()),
+        let pid = match qos {
+            QoS::AtMostOnce => None,
+            QoS::AtLeastOnce => pid.and_then(|p| Pid::try_from(p).ok()),
+            QoS::ExactlyOnce => pid.and_then(|p| Pid::try_from(p).ok()),
         };
 
         Publish {
-            qospid,
+            qos,
+            pid,
             payload,
             dup: false,
             retain: false,
@@ -580,8 +527,12 @@ mod test {
         }
     }
 
-    fn build_mqttstate() -> MqttState<Milliseconds, 128, 512> {
-        MqttState::new()
+    fn build_mqttstate() -> MqttState<Milliseconds> {
+        let state = MqttState::new();
+        const LEN: usize = 1024 * 10;
+        static mut PUBLISH_MEM: [u8; LEN] = [0u8; LEN];
+        BOXED_PUBLISH::grow(unsafe { &mut PUBLISH_MEM });
+        state
     }
 
     #[test]
@@ -591,80 +542,76 @@ mod test {
         let mut mqtt = build_mqttstate();
 
         // Publish
-        let publish = build_outgoing_publish(QoS::AtMostOnce);
+        let publish = Packet::Publish(build_publish(QoS::AtMostOnce, None));
+
+        let len = encode_slice(&publish, buf).unwrap();
 
         // Packet id shouldn't be set and publish shouldn't be saved in queue
-        let publish_out = match mqtt.handle_outgoing_request(publish.try_into().unwrap(), buf, &now)
-        {
-            Ok(Packet::Publish(p)) => p,
-            _ => panic!("Invalid packet. Should've been a publish packet"),
-        };
-        assert_eq!(publish_out.qospid, QosPid::AtMostOnce);
-        assert_eq!(mqtt.outgoing_pub.len(), 0);
+        mqtt.handle_outgoing_request(&mut SerializedPacket(&mut buf[..len]), &now)
+            .unwrap();
+        // assert_eq!(publish_out.qos, QoS::AtMostOnce);
+        // assert_eq!(mqtt.outgoing_pub.len(), 0);
 
-        // Subscribe
-        let subscribe = SubscribeRequest {
-            topics: Vec::from_slice(&[
-                SubscribeTopic {
-                    topic_path: String::from("some/topic"),
-                    qos: QoS::AtLeastOnce,
-                },
-                SubscribeTopic {
-                    topic_path: String::from("some/other/topic"),
-                    qos: QoS::ExactlyOnce,
-                },
-            ])
-            .unwrap(),
-        };
+        // // Subscribe
+        // let subscribe = SubscribeRequest {
+        //     topics: Vec::from_slice(&[
+        //         SubscribeTopic {
+        //             topic_path: String::from("some/topic"),
+        //             qos: QoS::AtLeastOnce,
+        //         },
+        //         SubscribeTopic {
+        //             topic_path: String::from("some/other/topic"),
+        //             qos: QoS::ExactlyOnce,
+        //         },
+        //     ])
+        //     .unwrap(),
+        // };
 
-        // Packet id should be set and subscribe shouldn't be saved in publish queue
-        let subscribe_out =
-            match mqtt.handle_outgoing_request(subscribe.try_into().unwrap(), buf, &now) {
-                Ok(Packet::Subscribe(p)) => p,
-                _ => panic!("Invalid packet. Should've been a subscribe packet"),
-            };
-        let mut topics_iter = subscribe_out.topics.iter();
+        // // Packet id should be set and subscribe shouldn't be saved in publish queue
+        // mqtt.handle_outgoing_request(subscribe.try_into().unwrap(), buf, &now)
+        //     .unwrap();
+        // let mut topics_iter = subscribe_out.topics.iter();
 
-        assert_eq!(subscribe_out.pid, Pid::try_from(2).unwrap());
-        assert_eq!(
-            topics_iter.next(),
-            Some(&SubscribeTopic {
-                qos: QoS::AtLeastOnce,
-                topic_path: String::from("some/topic")
-            })
-        );
-        assert_eq!(
-            topics_iter.next(),
-            Some(&SubscribeTopic {
-                qos: QoS::ExactlyOnce,
-                topic_path: String::from("some/other/topic")
-            })
-        );
-        assert_eq!(topics_iter.next(), None);
-        assert_eq!(mqtt.outgoing_pub.len(), 0);
+        // assert_eq!(subscribe_out.pid, Pid::try_from(2).unwrap());
+        // assert_eq!(
+        //     topics_iter.next(),
+        //     Some(&SubscribeTopic {
+        //         qos: QoS::AtLeastOnce,
+        //         topic_path: String::from("some/topic")
+        //     })
+        // );
+        // assert_eq!(
+        //     topics_iter.next(),
+        //     Some(&SubscribeTopic {
+        //         qos: QoS::ExactlyOnce,
+        //         topic_path: String::from("some/other/topic")
+        //     })
+        // );
+        // assert_eq!(topics_iter.next(), None);
+        // assert_eq!(mqtt.outgoing_pub.len(), 0);
 
-        // Unsubscribe
-        let unsubscribe = UnsubscribeRequest {
-            topics: Vec::from_slice(&[
-                String::from("some/topic"),
-                String::from("some/other/topic"),
-            ])
-            .unwrap(),
-        };
+        // // Unsubscribe
+        // let unsubscribe = UnsubscribeRequest {
+        //     topics: Vec::from_slice(&[
+        //         String::from("some/topic"),
+        //         String::from("some/other/topic"),
+        //     ])
+        //     .unwrap(),
+        // };
 
-        // Packet id should be set and subscribe shouldn't be saved in publish queue
-        let unsubscribe_out =
-            match mqtt.handle_outgoing_request(unsubscribe.try_into().unwrap(), buf, &now) {
-                Ok(Packet::Unsubscribe(p)) => p,
-                _ => panic!("Invalid packet. Should've been a unsubscribe packet"),
-            };
-        let mut topics_iter = unsubscribe_out.topics.iter();
+        // // Packet id should be set and subscribe shouldn't be saved in publish queue
+        // let unsubscribe_out =
+        //     match mqtt.handle_outgoing_request(unsubscribe.try_into().unwrap(), buf, &now) {
+        //         Ok(Packet::Unsubscribe(p)) => p,
+        //         _ => panic!("Invalid packet. Should've been a unsubscribe packet"),
+        //     };
+        // let mut topics_iter = unsubscribe_out.topics.iter();
 
-        assert_eq!(unsubscribe_out.pid, Pid::try_from(3).unwrap());
-        assert_eq!(topics_iter.next(), Some(&String::from("some/topic")));
-        assert_eq!(topics_iter.next(), Some(&String::from("some/other/topic")));
-        assert_eq!(topics_iter.next(), None);
-        assert_eq!(mqtt.outgoing_pub.len(), 0);
+        // assert_eq!(unsubscribe_out.pid, Pid::try_from(3).unwrap());
+        // assert_eq!(topics_iter.next(), Some(&String::from("some/topic")));
+        // assert_eq!(topics_iter.next(), Some(&String::from("some/other/topic")));
+        // assert_eq!(topics_iter.next(), None);
+        // assert_eq!(mqtt.outgoing_pub.len(), 0);
     }
 
     #[test]
@@ -675,65 +622,34 @@ mod test {
         let mut mqtt = build_mqttstate();
 
         // QoS0 Publish
-        let publish = build_outgoing_publish(QoS::AtMostOnce);
+        let publish = Packet::Publish(build_publish(QoS::AtMostOnce, None));
+        let len = encode_slice(&publish, buf).unwrap();
+        let mut pkg = SerializedPacket(&mut buf[..len]);
 
         // Packet id shouldn't be set and publish shouldn't be saved in queue
-        let publish_out = match mqtt.handle_outgoing_publish(publish, buf, &now) {
-            Ok(Packet::Publish(p)) => p,
-            _ => panic!("Invalid packet. Should've been a publish packet"),
+        mqtt.handle_outgoing_publish(&mut pkg, &now).unwrap();
+
+        let publish_out = match decode_slice(pkg.to_inner()).unwrap() {
+            Some(Packet::Publish(p)) => p,
+            _ => panic!(),
         };
-        assert_eq!(publish_out.qospid, QosPid::AtMostOnce);
+        assert_eq!(publish_out.qos, QoS::AtMostOnce);
         assert_eq!(mqtt.outgoing_pub.len(), 0);
 
         // QoS1 Publish
-        let publish = build_outgoing_publish(QoS::AtLeastOnce);
+        let publish = Packet::Publish(build_publish(QoS::AtLeastOnce, None));
+        let len = encode_slice(&publish, buf).unwrap();
+        let mut pkg = SerializedPacket(&mut buf[..len]);
 
         // Packet id should be set and publish should be saved in queue
-        let publish_out = match mqtt.handle_outgoing_publish(publish.clone(), buf, &now) {
-            Ok(Packet::Publish(p)) => p,
-            _ => panic!("Invalid packet. Should've been a publish packet"),
+        mqtt.handle_outgoing_publish(&mut pkg, &now).unwrap();
+        let publish_out = match decode_slice(pkg.to_inner()).unwrap() {
+            Some(Packet::Publish(p)) => p,
+            _ => panic!(),
         };
-        assert_eq!(
-            publish_out.qospid,
-            QosPid::AtLeastOnce(Pid::try_from(2).unwrap())
-        );
+        assert_eq!(publish_out.qos, QoS::AtLeastOnce);
+        assert_eq!(publish_out.pid, Some(Pid::try_from(2).unwrap()));
         assert_eq!(mqtt.outgoing_pub.len(), 1);
-
-        // Packet id should be incremented and publish should be saved in queue
-        let publish_out = match mqtt.handle_outgoing_publish(publish.clone(), buf, &now) {
-            Ok(Packet::Publish(p)) => p,
-            _ => panic!("Invalid packet. Should've been a publish packet"),
-        };
-        assert_eq!(
-            publish_out.qospid,
-            QosPid::AtLeastOnce(Pid::try_from(3).unwrap())
-        );
-        assert_eq!(mqtt.outgoing_pub.len(), 2);
-
-        // QoS1 Publish
-        let publish = build_outgoing_publish(QoS::ExactlyOnce);
-
-        // Packet id should be set and publish should be saved in queue
-        let publish_out = match mqtt.handle_outgoing_publish(publish.clone(), buf, &now) {
-            Ok(Packet::Publish(p)) => p,
-            _ => panic!("Invalid packet. Should've been a publish packet"),
-        };
-        assert_eq!(
-            publish_out.qospid,
-            QosPid::ExactlyOnce(Pid::try_from(4).unwrap())
-        );
-        assert_eq!(mqtt.outgoing_pub.len(), 3);
-
-        // Packet id should be incremented and publish should be saved in queue
-        let publish_out = match mqtt.handle_outgoing_publish(publish.clone(), buf, &now) {
-            Ok(Packet::Publish(p)) => p,
-            _ => panic!("Invalid packet. Should've been a publish packet"),
-        };
-        assert_eq!(
-            publish_out.qospid,
-            QosPid::ExactlyOnce(Pid::try_from(5).unwrap())
-        );
-        assert_eq!(mqtt.outgoing_pub.len(), 4);
     }
 
     #[test]
@@ -741,9 +657,9 @@ mod test {
         let mut mqtt = build_mqttstate();
 
         // QoS0, 1, 2 Publishes
-        let publish1 = build_incoming_publish(QoS::AtMostOnce, 1);
-        let publish2 = build_incoming_publish(QoS::AtLeastOnce, 2);
-        let publish3 = build_incoming_publish(QoS::ExactlyOnce, 3);
+        let publish1 = build_publish(QoS::AtMostOnce, Some(1));
+        let publish2 = build_publish(QoS::AtLeastOnce, Some(2));
+        let publish3 = build_publish(QoS::ExactlyOnce, Some(3));
 
         mqtt.handle_incoming_publish(publish1).unwrap();
         mqtt.handle_incoming_publish(publish2).unwrap();
@@ -757,15 +673,12 @@ mod test {
     #[test]
     fn incoming_qos2_publish_should_send_rec_to_network_and_publish_to_user() {
         let mut mqtt = build_mqttstate();
-        let publish = build_incoming_publish(QoS::ExactlyOnce, 1);
+        let publish = build_publish(QoS::ExactlyOnce, Some(1));
 
         let (notification, request) = mqtt.handle_incoming_publish(publish).unwrap();
 
         match notification {
-            Some(Notification::Publish(publish)) => assert_eq!(
-                publish.qospid,
-                QosPid::ExactlyOnce(Pid::try_from(1).unwrap())
-            ),
+            Some(Notification::Publish(publish)) => assert_eq!(publish.qospid, QoS::ExactlyOnce),
             _ => panic!("Invalid notification: {:?}", notification),
         }
 
@@ -781,20 +694,21 @@ mod test {
         let buf = &mut [0u8; 256];
         let now = 0u32.milliseconds();
 
-        let publish1 = build_outgoing_publish(QoS::AtLeastOnce);
-        let publish2 = build_outgoing_publish(QoS::ExactlyOnce);
+        let publish1 = Packet::Publish(build_publish(QoS::AtLeastOnce, None));
+        let len = encode_slice(&publish1, buf).unwrap();
+        let mut pkg1 = SerializedPacket(&mut buf[..len]);
+        mqtt.handle_outgoing_publish(&mut pkg1, &now).unwrap();
 
-        mqtt.handle_outgoing_publish(publish1, buf, &now).unwrap();
-        mqtt.handle_outgoing_publish(publish2, buf, &now).unwrap();
-
-        mqtt.handle_incoming_puback(Pid::try_from(2).unwrap())
-            .unwrap();
         assert_eq!(mqtt.outgoing_pub.len(), 1);
 
-        let backup = mqtt.outgoing_pub.get(&3);
-        assert_eq!(backup.unwrap().publish.qos, QoS::ExactlyOnce);
+        let backup = mqtt.outgoing_pub.get_mut(&2).unwrap().packet(1).unwrap();
+        let publish_out = match decode_slice(backup).unwrap() {
+            Some(Packet::Publish(p)) => p,
+            _ => panic!(),
+        };
+        assert_eq!(publish_out.qos, QoS::AtLeastOnce);
 
-        mqtt.handle_incoming_puback(Pid::try_from(3).unwrap())
+        mqtt.handle_incoming_puback(Pid::try_from(2).unwrap())
             .unwrap();
         assert_eq!(mqtt.outgoing_pub.len(), 0);
     }
@@ -805,24 +719,18 @@ mod test {
         let buf = &mut [0u8; 256];
         let now = 0u32.milliseconds();
 
-        let publish1 = build_outgoing_publish(QoS::AtLeastOnce);
-        let publish2 = build_outgoing_publish(QoS::ExactlyOnce);
+        let publish = Packet::Publish(build_publish(QoS::ExactlyOnce, None));
+        let len = encode_slice(&publish, buf).unwrap();
+        let mut pkg = SerializedPacket(&mut buf[..len]);
+        mqtt.handle_outgoing_publish(&mut pkg, &now).unwrap();
 
-        mqtt.handle_outgoing_publish(publish1, buf, &now).unwrap();
-        mqtt.handle_outgoing_publish(publish2, buf, &now).unwrap();
-
-        mqtt.handle_incoming_pubrec(Pid::try_from(3).unwrap())
+        mqtt.handle_incoming_pubrec(Pid::try_from(2).unwrap())
             .unwrap();
-        assert_eq!(mqtt.outgoing_pub.len(), 1);
-
-        // check if the remaining element's pid is 2
-        let backup = mqtt.outgoing_pub.get(&2);
-        assert_eq!(backup.unwrap().publish.qos, QoS::AtLeastOnce);
-
+        assert_eq!(mqtt.outgoing_pub.len(), 0);
         assert_eq!(mqtt.outgoing_rel.len(), 1);
 
-        // check if the  element's pid is 3
-        assert!(mqtt.outgoing_rel.contains(&3));
+        // check if the  element's pid is 2
+        assert!(mqtt.outgoing_rel.contains(&2));
     }
 
     #[test]
@@ -833,8 +741,10 @@ mod test {
         let pid = Pid::try_from(2).unwrap();
         assert_eq!(pid.get(), 2);
 
-        let publish = build_outgoing_publish(QoS::ExactlyOnce);
-        mqtt.handle_outgoing_publish(publish, buf, &now).unwrap();
+        let publish = Packet::Publish(build_publish(QoS::ExactlyOnce, None));
+        let len = encode_slice(&publish, buf).unwrap();
+        let mut pkg = SerializedPacket(&mut buf[..len]);
+        mqtt.handle_outgoing_publish(&mut pkg, &now).unwrap();
 
         let (notification, request) = mqtt.handle_incoming_pubrec(pid).unwrap();
 
@@ -845,7 +755,7 @@ mod test {
     #[test]
     fn incoming_pubrel_should_send_comp_to_network_and_nothing_to_user() {
         let mut mqtt = build_mqttstate();
-        let publish = build_incoming_publish(QoS::ExactlyOnce, 1);
+        let publish = build_publish(QoS::ExactlyOnce, Some(1));
 
         let pid = Pid::try_from(1).unwrap();
         assert_eq!(pid.get(), 1);
@@ -862,10 +772,13 @@ mod test {
         let mut mqtt = build_mqttstate();
         let buf = &mut [0u8; 256];
         let now = 0u32.milliseconds();
-        let publish = build_outgoing_publish(QoS::ExactlyOnce);
+        let publish = Packet::Publish(build_publish(QoS::ExactlyOnce, None));
+        let len = encode_slice(&publish, buf).unwrap();
+        let mut pkg = SerializedPacket(&mut buf[..len]);
+
         let pid = Pid::try_from(2).unwrap();
 
-        mqtt.handle_outgoing_publish(publish, buf, &now).unwrap();
+        mqtt.handle_outgoing_publish(&mut pkg, &now).unwrap();
         mqtt.handle_incoming_pubrec(pid).unwrap();
 
         mqtt.handle_incoming_pubcomp(pid).unwrap();
@@ -882,9 +795,11 @@ mod test {
         assert!(mqtt.await_pingresp);
 
         // network activity other than pingresp
-        let publish = build_outgoing_publish(QoS::AtLeastOnce);
-        mqtt.handle_outgoing_publish(publish.into(), buf, &now)
-            .unwrap();
+        let publish = Packet::Publish(build_publish(QoS::AtLeastOnce, None));
+        let len = encode_slice(&publish, buf).unwrap();
+        let mut pkg = SerializedPacket(&mut buf[..len]);
+
+        mqtt.handle_outgoing_publish(&mut pkg, &now).unwrap();
         mqtt.handle_incoming_packet(Packet::Puback(Pid::try_from(2).unwrap()))
             .unwrap();
 
